# ca_pump: calcium pump example morph

# this is a really challenging conformational morph, but the input
# structures are (nearly?) identical with respect to sequence and
# identifiers so the prep work is quick!

# NOTE: the refinement step for 7k+ atoms is a bit time consuming --
# probably 5-25 minutes total CPU time, depending on your system.
# -- but it is well worth the wait!


# load the input conformations
# (from local PDB files if present...otherwise fetched via the web)

fetch 2agv, conf1, async=0

fetch 1su4, conf2, async=0


# only work with a single copy

remove not chain A


# perform a deep, core-seeking structural alignment

align conf2, conf1, cutoff=1.5, cycles=20


# create a sequence alignment (without superposition)

align conf2, conf1, object=seq_aln, cycles=0, transform=0


# force a refresh (to create the seq_aln selection)

refresh


# create a hybrid object using only the aligned atoms

create morph_inp, conf1 and seq_aln, 1, 1

create morph_inp, conf2 and seq_aln, 1, 2


# make it pretty

dss morph_inp

as cartoon, morph_inp

spectrum count, selection=morph_inp and elem c

show lines, morph_inp

set cartoon_side_chain_helper

dss morph_inp

 
# create the morph (runs asynch so you can watch)

from epymol import rigimol

rigimol.morph("morph_inp","morph_out",refinement=5, async=1)

